//System clock frequency definition
#define FCY (70000000)

//Device related headers
#include <p33EP128GP504.h>
#include <libpic30.h>


//Global variables used by module App_TempCalc for read:
extern int App_Main_I_pump;
extern unsigned int  App_Main_V_Supply;
extern unsigned char App_HeaterControl_HeaterState;
extern unsigned char App_HeaterControl_OpPointReached;
extern unsigned char App_PumpControl_OpPointReached;

//Global variables used by module App_TempCalc for write:
extern int App_AFRLambdaCalc_AFRDetected;
extern int App_AFRLambdaCalc_LambdaDetectedFilt;
extern int App_AFRLambdaCalc_LambdaDetected;
extern int App_AFRLambdaCalc_I_pump_corr;

//External Calibrations declarations:
extern int CAL_CleanAirPumpRatio;
extern int CAL_SensorParasiticOffset;
extern const int CAL_TBL_Ip_Lambda[2][24];
extern const int CAL_TBL_Ip_AFR[2][24];

//Externally used functions prototypes
int App_Interpolation_Linear_si16 (int x, int size, const int array[2][size/2]);

//This function calculates the lambda and AFR ratio including error handling
void App_AFRLambdaCalc (void)
{
    if (App_Main_I_pump > 0) //If in the lean region
    {
        //Compute the corrected Ipump using clean air calibration
        App_AFRLambdaCalc_I_pump_corr = __builtin_divsd(__builtin_mulss(App_Main_I_pump, CAL_CleanAirPumpRatio), 10000);
    }
    else
    {
        //Clean air calibration is not used in rich region
        App_AFRLambdaCalc_I_pump_corr = App_Main_I_pump;
    }
    
    //Is the warming-up phase in progress?
    if (App_HeaterControl_HeaterState < 5)
    {
        //Yes, output errorcode -1
        App_AFRLambdaCalc_LambdaDetected = -1;
        App_AFRLambdaCalc_LambdaDetectedFilt = -1;
        App_AFRLambdaCalc_AFRDetected = -1;
    }
    else
    {
        //No, then test if sensor cell overheated?
        if (App_HeaterControl_HeaterState > 5)
        {
            //Yes,  output errorcode -2
            App_AFRLambdaCalc_LambdaDetected = -2;
            App_AFRLambdaCalc_LambdaDetectedFilt = -2;
            App_AFRLambdaCalc_AFRDetected = -2;
        }
        else
        {
            //No, then test if temperature operation point is reached?
            if (!App_HeaterControl_OpPointReached)
            {
                //No, output errorcode -3
                App_AFRLambdaCalc_LambdaDetected = -3;
                App_AFRLambdaCalc_LambdaDetectedFilt = -3;
                App_AFRLambdaCalc_AFRDetected = -3;
            }
            else
            {
                //Yes, then test if Nernst cell operation point is reached?
                if (!App_PumpControl_OpPointReached)
                {
                    //No,  output errorcode -4
                    App_AFRLambdaCalc_LambdaDetected = -4;
                    App_AFRLambdaCalc_LambdaDetectedFilt = -4;
                    App_AFRLambdaCalc_AFRDetected = -4;
                }
                else
                {
                    //Yes, test if supply voltage is lower than 7V?
                    if (App_Main_V_Supply < 7000U)
                    {
                        //Yes, output errorcode -5;
                        App_AFRLambdaCalc_LambdaDetected = -5;
                        App_AFRLambdaCalc_LambdaDetectedFilt = -5;
                        App_AFRLambdaCalc_AFRDetected = -5;
                    }
                    else
                    {
                        //Calculate the Lambda and AFR using Ip/Lambda and Ip/AFR characteristic tables
                        App_AFRLambdaCalc_LambdaDetected = (App_Interpolation_Linear_si16(App_AFRLambdaCalc_I_pump_corr, 
                                                            (sizeof(CAL_TBL_Ip_Lambda) / sizeof(CAL_TBL_Ip_Lambda[0][0])), CAL_TBL_Ip_Lambda)
                                                            + CAL_SensorParasiticOffset);
                        //Perform a filter for Lambda
                        static int LambdaHistory[10];
                        long sum = 0;
                        int i;
                        for (i = 9; i >= 1; i--)
                        {
                            LambdaHistory[i] = LambdaHistory[i-1];
                        }
                        LambdaHistory[0] = App_AFRLambdaCalc_LambdaDetected;
                        for (i = 0; i < 10; i++)
                        {
                            sum += LambdaHistory[i];
                        }
                        App_AFRLambdaCalc_LambdaDetectedFilt = (__builtin_divsd(sum, 10));
                        App_AFRLambdaCalc_AFRDetected = App_Interpolation_Linear_si16 (App_AFRLambdaCalc_I_pump_corr, (sizeof(CAL_TBL_Ip_AFR) / sizeof(CAL_TBL_Ip_AFR[0][0])), CAL_TBL_Ip_AFR);
                    }
                }
            }
        }
    }
}