/*****************************************************************************/
// TFile contains RS-232 Diagnostic & service communication protocol
/*****************************************************************************/

//Diagnostic services:
//01 - Read data by ID
//FD - Routine control service
//FE - IOCBI mode enable / disable
//FF - IOCBI mode data

#include <p33EP128GP504.h>
#include <libpic30.h>
#include <xc.h>
#include <dsp.h>
//Hardware driver header
#include "../HWD/HWD.h"

//Global variables used by module App_DiagComDrv for read:
extern unsigned int  App_DiagComDrv_UART1RXBuf[8];
extern unsigned int  App_RiCalc_RiFilt;
extern unsigned int  App_Main_V_Nernst;
extern unsigned int  App_Main_V_RiH;
extern unsigned int  App_Main_V_RiL;
extern unsigned int  App_Main_V_Supply;
extern unsigned int  App_HeaterControl_DesiredPWMDutyCycle;
extern unsigned char App_HeaterControl_HeaterState;
extern int           App_TempCalc_CellTemp;
extern unsigned long App_OS_SystemTime_Sec;
extern int           App_AFRLambdaCalc_I_pump_corr;
extern int           App_Main_I_pump;
extern unsigned char App_HeaterControl_OpPointReached;
extern unsigned int  App_PumpControl_U_pump_Target;
extern int           App_AFRLambdaCalc_LambdaDetected;
extern int           App_AFRLambdaCalc_LambdaDetectedFilt;
extern int           App_AFRLambdaCalc_AFRDetected;

//Global variables used by module App_DiagComDrv for write:
extern unsigned int App_DiagComDrv_UART1TXBuf[8];
extern unsigned int App_DiagComDrv_IOCBI_Data_HeaterPWMDutyCycle;
extern unsigned int App_DiagComDrv_IOCBI_Enable_HeaterPWMDutyCycle;
extern unsigned int App_DiagComDrv_IOCBI_Data_HeaterPID_TargetTemp;
extern unsigned int App_DiagComDrv_IOCBI_Enable_HeaterPID_TargetTemp;
extern unsigned int App_DiagComDrv_IOCBI_Enable_U_pump_Target;
extern unsigned int App_DiagComDrv_IOCBI_Data_U_pump_Target;
extern unsigned int App_DiagComDrv_IOCBI_Enable_U_OutDAC;
extern unsigned int App_DiagComDrv_IOCBI_Data_U_OutDAC;
extern tPID App_HeaterControl_PIDStc;
extern tPID App_PumpControl_PIDStc;
extern fractional App_HeaterControl_kCoeffs[3];
extern fractional App_PumpControl_kCoeffs[3];
extern int CAL_CleanAirPumpRatio;

//Local functions prototypes
static void Diag_Return_NRC_IMLOIF (void);
static void Diag_Return_NRC_ROOR (void);
static void Diag_Return_PRC_S22 (unsigned int DataH, unsigned int DataL);
static void Diag_Return_PRC_S31 (void);
static void Diag_Return_PRC_S2F (void);
static void Diag_Return_NRC_SNS (void);
static void Diag_RDBI_S22_0001 (void);
static void Diag_RDBI_S22_0100 (void);
static void Diag_RDBI_S22_0101 (void);
static void Diag_RDBI_S22_0102 (void);
static void Diag_RDBI_S22_0103 (void);
static void Diag_RDBI_S22_0104 (void);
static void Diag_RDBI_S22_0105 (void);
static void Diag_RDBI_S22_0106 (void);
static void Diag_RDBI_S22_0107 (void);
static void Diag_RDBI_S22_0108 (void);
static void Diag_RDBI_S22_0109 (void);
static void Diag_RDBI_S22_010A (void);
static void Diag_RDBI_S22_010B (void);
static void Diag_RDBI_S22_010C (void);
static void Diag_RDBI_S22_010D (void);
static void Diag_RDBI_S22_010E (void);
static void Diag_RDBI_S22_010F (void);
static void Diag_RDBI_S22_0110 (void);
static void Diag_IOCBI_S2F_0F00 (void);
static void Diag_IOCBI_S2F_0F01 (void);
static void Diag_IOCBI_S2F_0F02 (void);
static void Diag_IOCBI_S2F_0F03 (void);
static void Diag_RC_S31_0001 (void);
static void Diag_RC_S31_0002 (void);
static void Diag_RC_S31_0003 (void);
static void Diag_RC_S31_0004 (void);
static void Diag_RC_S31_0005 (void);
static void Diag_RC_S31_0006 (void);
static void Diag_RC_S31_000A (void);


void App_DiagComDrv(void)
{
    if(HWD_CheckUARTRXBuffer() == 1)
    {
        //Define a local 16-bit signal for storage ID
	    unsigned short int loc_ID = (unsigned short int)((App_DiagComDrv_UART1RXBuf[2] << 8) | App_DiagComDrv_UART1RXBuf[3]);

	    //Switch the requested diagnostic service
	    switch (App_DiagComDrv_UART1RXBuf[1])
	    {
	    	case 0x22: //Service 22 requested
	    		switch (loc_ID) //Switch the requested ID
	    		{
	    			case 0x0001://ECU identification
	    				Diag_RDBI_S22_0001();
	    				break;
                    case 0x0100://Detected Lambda Filtered
	    				Diag_RDBI_S22_0100();
	    				break;
                    case 0x0101://Pump current corrected
	    				Diag_RDBI_S22_0101();
	    				break;
                    case 0x0102://Cell temperature
	    				Diag_RDBI_S22_0102();
	    				break;
                    case 0x0103://Nernst cell internal resistance
	    				Diag_RDBI_S22_0103();
	    				break;
                    case 0x0104://Heater duty cycle
	    				Diag_RDBI_S22_0104();
	    				break;
                    case 0x0105://Supply voltage
	    				Diag_RDBI_S22_0105();
	    				break;
                    case 0x0106://Nernst voltage
	    				Diag_RDBI_S22_0106();
	    				break;
                    case 0x0107://Nernst voltage positive pulse
	    				Diag_RDBI_S22_0107();
                        break;
                    case 0x0108://Nernst voltage negative pulse
	    				Diag_RDBI_S22_0108();
                        break;
                    case 0x0109://Heater controller state
	    				Diag_RDBI_S22_0109();
                        break;
                    case 0x010A://System time
	    				Diag_RDBI_S22_010A();
                        break;
                    case 0x010B://Target pumping voltage
	    				Diag_RDBI_S22_010B();
                        break;
                    case 0x010C://Heater operation point flag
	    				Diag_RDBI_S22_010C();
                        break;
                    case 0x010D://Detected AFR for gasoline
	    				Diag_RDBI_S22_010D();
                        break;
                    case 0x010E://Pump current uncorrected
	    				Diag_RDBI_S22_010E();
                        break;
                    case 0x010F://Detected Lambda unfiltered
	    				Diag_RDBI_S22_010F();
                        break;
                    case 0x0110://Clean air calibration
	    				Diag_RDBI_S22_0110();
                        break;
	    			default: //ID does not exist, NRC ROOR
	    				Diag_Return_NRC_ROOR();
	    				break;
	    		}
	    		break;

	    	case 0x2E: //Service 2E requested
	    		switch (loc_ID)
	    		{
	    		    case 0xF000:
	    		    	//Diag_WDBI_S2E_F000();
	    		    	break;
	    		    case 0xF001:
	    		    	//Diag_WDBI_S2E_F001();
	    		    	break;
                    default: //ID does not exist, NRC ROOR
	    			    Diag_Return_NRC_ROOR();
	    			    break;
	    		}
	    		break;

	    	case 0x2F: //Service 2F requested
	    		switch (loc_ID)
	    		{
	    	    	case 0x0F00: //Force heater PWM duty cycle
	    	    		Diag_IOCBI_S2F_0F00();
	    	    		break;
                    case 0x0F01: //Force heater PID target temperature
	    	    		Diag_IOCBI_S2F_0F01();
	    	    		break;
	    	    	case 0x0F02: //Force pump current
	    	    		Diag_IOCBI_S2F_0F02();
	    	    		break;
	    	    	case 0x0F03: //Force analog output DAC code 
	    	    		Diag_IOCBI_S2F_0F03();
	    	    		break;
	    		    default: //ID does not exist, NRC ROOR
	    			    Diag_Return_NRC_ROOR();
	    			    break;
	    		}
	    		break;

	    	case 0x31: //Service 31 requested
	    		switch (loc_ID)
	    		{
                    case 0x0001: //id 0001 - Set the P-parameter of heater PID controller
                        Diag_RC_S31_0001();
                        break;
                    case 0x0002: //id 0002 - Set the I-parameter of heater PID controller
                        Diag_RC_S31_0002();
                        break;
                    case 0x0003: //id 0003 - Set the D-parameter of heater PID controller
                        Diag_RC_S31_0003();
                        break;
                    case 0x0004: //id 0004 - Set the P-parameter of pump PID controller
                        Diag_RC_S31_0004();
                        break;
                    case 0x0005: //id 0005 - Set the I-parameter of pump PID controller
                        Diag_RC_S31_0005();
                        break;
                    case 0x0006: //id 0006 - Set the D-parameter of pump PID controller
                        Diag_RC_S31_0006();
                        break;
                    case 0x000A: //id 000A - Clean air calibration
                        Diag_RC_S31_000A();
                        break;
	    			default: //ID does not exist, NRC ROOR
	    				Diag_Return_NRC_ROOR();
	    				break;
	    		}
	    		break;
	    	default:
	    		Diag_Return_NRC_SNS();
	    		break;
	    }
        //Return the response - set the DMA to send the TX buffer
        DMA0CONbits.CHEN = 1; // Enable the DMA channel
        DMA0REQbits.FORCE = 1;
    }
}

static void Diag_RDBI_S22_0001 (void)
{
	if (App_DiagComDrv_UART1RXBuf[0] == 3)
	{
        unsigned char id[4] = {'W','B','O','2'}; 
	    Diag_Return_PRC_S22((unsigned int)((id[0])|(id[1]<<8)), (unsigned int)((id[2])|(id[3]<<8)));
	}
	else
	{
		Diag_Return_NRC_IMLOIF();
	}
}

static void Diag_RDBI_S22_0100 (void)
{
	if (App_DiagComDrv_UART1RXBuf[0] == 3)
	{
	    Diag_Return_PRC_S22(0, App_AFRLambdaCalc_LambdaDetectedFilt);
	}
	else
	{
		Diag_Return_NRC_IMLOIF();
	}
}

static void Diag_RDBI_S22_0101 (void)
{
	if (App_DiagComDrv_UART1RXBuf[0] == 3)
	{
	    Diag_Return_PRC_S22(0, App_AFRLambdaCalc_I_pump_corr);
	}
	else
	{
		Diag_Return_NRC_IMLOIF();
	}
}

static void Diag_RDBI_S22_0102 (void)
{
	if (App_DiagComDrv_UART1RXBuf[0] == 3)
	{
	    Diag_Return_PRC_S22(0, App_TempCalc_CellTemp);
	}
	else
	{
		Diag_Return_NRC_IMLOIF();
	}
}

static void Diag_RDBI_S22_0103 (void)
{
	if (App_DiagComDrv_UART1RXBuf[0] == 3)
	{
	    Diag_Return_PRC_S22(0, App_RiCalc_RiFilt);
	}
	else
	{
		Diag_Return_NRC_IMLOIF();
	}
}

static void Diag_RDBI_S22_0104 (void)
{
	if (App_DiagComDrv_UART1RXBuf[0] == 3)
	{
	    Diag_Return_PRC_S22(0, App_HeaterControl_DesiredPWMDutyCycle);
	}
	else
	{
		Diag_Return_NRC_IMLOIF();
	}
}

static void Diag_RDBI_S22_0105 (void)
{
	if (App_DiagComDrv_UART1RXBuf[0] == 3)
	{
	    Diag_Return_PRC_S22(0, App_Main_V_Supply);
	}
	else
	{
		Diag_Return_NRC_IMLOIF();
	}
}

static void Diag_RDBI_S22_0106 (void)
{
	if (App_DiagComDrv_UART1RXBuf[0] == 3)
	{
	    Diag_Return_PRC_S22(0, App_Main_V_Nernst);
	}
	else
	{
		Diag_Return_NRC_IMLOIF();
	}
}

static void Diag_RDBI_S22_0107 (void)
{
	if (App_DiagComDrv_UART1RXBuf[0] == 3)
	{
	    Diag_Return_PRC_S22(0, App_Main_V_RiH);
	}
	else
	{
		Diag_Return_NRC_IMLOIF();
	}
}

static void Diag_RDBI_S22_0108 (void)
{
	if (App_DiagComDrv_UART1RXBuf[0] == 3)
	{
	    Diag_Return_PRC_S22(0, App_Main_V_RiL);
	}
	else
	{
		Diag_Return_NRC_IMLOIF();
	}
}

static void Diag_RDBI_S22_0109 (void)
{
	if (App_DiagComDrv_UART1RXBuf[0] == 3)
	{
	    Diag_Return_PRC_S22(0, App_HeaterControl_HeaterState);
	}
	else
	{
		Diag_Return_NRC_IMLOIF();
	}
}

static void Diag_RDBI_S22_010A (void)
{
	if (App_DiagComDrv_UART1RXBuf[0] == 3)
	{
	    Diag_Return_PRC_S22(0, App_OS_SystemTime_Sec);
	}
	else
	{
		Diag_Return_NRC_IMLOIF();
	}
}

static void Diag_RDBI_S22_010B (void)
{
	if (App_DiagComDrv_UART1RXBuf[0] == 3)
	{
	    Diag_Return_PRC_S22(0, App_PumpControl_U_pump_Target);
	}
	else
	{
		Diag_Return_NRC_IMLOIF();
	}
}

static void Diag_RDBI_S22_010C (void)
{
	if (App_DiagComDrv_UART1RXBuf[0] == 3)
	{
	    Diag_Return_PRC_S22(0, App_HeaterControl_OpPointReached);
	}
	else
	{
		Diag_Return_NRC_IMLOIF();
	}
}

static void Diag_RDBI_S22_010D (void)
{
	if (App_DiagComDrv_UART1RXBuf[0] == 3)
	{
	    Diag_Return_PRC_S22(0, App_AFRLambdaCalc_AFRDetected);
	}
	else
	{
		Diag_Return_NRC_IMLOIF();
	}
}

static void Diag_RDBI_S22_010E (void)
{
	if (App_DiagComDrv_UART1RXBuf[0] == 3)
	{
	    Diag_Return_PRC_S22(0, App_Main_I_pump);
	}
	else
	{
		Diag_Return_NRC_IMLOIF();
	}
}

static void Diag_RDBI_S22_010F (void)
{
	if (App_DiagComDrv_UART1RXBuf[0] == 3)
	{
	    Diag_Return_PRC_S22(0, App_AFRLambdaCalc_LambdaDetected);
	}
	else
	{
		Diag_Return_NRC_IMLOIF();
	}
}

static void Diag_RDBI_S22_0110 (void)
{
	if (App_DiagComDrv_UART1RXBuf[0] == 3)
	{
	    Diag_Return_PRC_S22(0, CAL_CleanAirPumpRatio);
	}
	else
	{
		Diag_Return_NRC_IMLOIF();
	}
}

static void Diag_IOCBI_S2F_0F00 (void)
{
	if (App_DiagComDrv_UART1RXBuf[0] == 7)
	{
      App_DiagComDrv_IOCBI_Data_HeaterPWMDutyCycle = (unsigned int)(App_DiagComDrv_UART1RXBuf[6] | (App_DiagComDrv_UART1RXBuf[7] << 8));
      if (App_DiagComDrv_IOCBI_Data_HeaterPWMDutyCycle < 10001)
      {
        App_DiagComDrv_IOCBI_Enable_HeaterPWMDutyCycle = 0x55AA;
      }
      else
      {
        App_DiagComDrv_IOCBI_Enable_HeaterPWMDutyCycle = 0;
      }
      Diag_Return_PRC_S2F();
    }
	else
	{
		Diag_Return_NRC_IMLOIF();
	}
}

static void Diag_IOCBI_S2F_0F01 (void)
{
	if (App_DiagComDrv_UART1RXBuf[0] == 7)
	{
      App_DiagComDrv_IOCBI_Data_HeaterPID_TargetTemp = (unsigned int)(App_DiagComDrv_UART1RXBuf[6] | (App_DiagComDrv_UART1RXBuf[7] << 8));
      if (App_DiagComDrv_IOCBI_Data_HeaterPID_TargetTemp < 8500)
      {
        App_DiagComDrv_IOCBI_Enable_HeaterPID_TargetTemp = 0x55AA;
      }
      else
      {
        App_DiagComDrv_IOCBI_Enable_HeaterPID_TargetTemp = 0;
      }
      Diag_Return_PRC_S2F();
    }
	else
	{
		Diag_Return_NRC_IMLOIF();
	}
}

static void Diag_IOCBI_S2F_0F02 (void)
{
	if (App_DiagComDrv_UART1RXBuf[0] == 7)
	{
      App_DiagComDrv_IOCBI_Data_U_pump_Target = (unsigned int)(App_DiagComDrv_UART1RXBuf[6] | (App_DiagComDrv_UART1RXBuf[7] << 8));
      if (App_DiagComDrv_IOCBI_Data_U_pump_Target <= 4095)
      {
        App_DiagComDrv_IOCBI_Enable_U_pump_Target = 0x55AA;
      }
      else
      {
        App_DiagComDrv_IOCBI_Enable_U_pump_Target = 0;
      }
      Diag_Return_PRC_S2F();
    }
	else
	{
		Diag_Return_NRC_IMLOIF();
	}
}

static void Diag_IOCBI_S2F_0F03 (void)
{
	if (App_DiagComDrv_UART1RXBuf[0] == 7)
	{
      App_DiagComDrv_IOCBI_Data_U_OutDAC = (unsigned int)(App_DiagComDrv_UART1RXBuf[6] | (App_DiagComDrv_UART1RXBuf[7] << 8));
      if (App_DiagComDrv_IOCBI_Data_U_OutDAC <= 4095)
      {
        App_DiagComDrv_IOCBI_Enable_U_OutDAC = 0x55AA;
      }
      else
      {
        App_DiagComDrv_IOCBI_Enable_U_OutDAC = 0;
      }
      Diag_Return_PRC_S2F();
    }
	else
	{
		Diag_Return_NRC_IMLOIF();
	}
}

static void Diag_RC_S31_0001 (void)
{
	if (App_DiagComDrv_UART1RXBuf[0] == 5)
	{
        PIDInit(&App_HeaterControl_PIDStc); //This clears the controller history
        App_HeaterControl_kCoeffs[0] = (App_DiagComDrv_UART1RXBuf[4] | (App_DiagComDrv_UART1RXBuf[5] << 8)); //Load the P - coefficient
        PIDCoeffCalc(&App_HeaterControl_kCoeffs[0], &App_HeaterControl_PIDStc); /*Derive the a,b, & c coefficients from the Kp, Ki & Kd */
	    Diag_Return_PRC_S31();
	}
	else
	{
		Diag_Return_NRC_IMLOIF();
	}
}

static void Diag_RC_S31_0002 (void)
{
	if (App_DiagComDrv_UART1RXBuf[0] == 5)
	{
        PIDInit(&App_HeaterControl_PIDStc); //This clears the controller history
        App_HeaterControl_kCoeffs[1] = (App_DiagComDrv_UART1RXBuf[4] | (App_DiagComDrv_UART1RXBuf[5] << 8)); //Load the I - coefficient
        PIDCoeffCalc(&App_HeaterControl_kCoeffs[0], &App_HeaterControl_PIDStc); /*Derive the a,b, & c coefficients from the Kp, Ki & Kd */
	    Diag_Return_PRC_S31();
	}
	else
	{
		Diag_Return_NRC_IMLOIF();
	}
}

static void Diag_RC_S31_0003 (void)
{
	if (App_DiagComDrv_UART1RXBuf[0] == 5)
	{
        PIDInit(&App_HeaterControl_PIDStc); //This clears the controller history
        App_HeaterControl_kCoeffs[2] = (App_DiagComDrv_UART1RXBuf[4] | (App_DiagComDrv_UART1RXBuf[5] << 8)); //Load the D - coefficient
        PIDCoeffCalc(&App_HeaterControl_kCoeffs[0], &App_HeaterControl_PIDStc); /*Derive the a,b, & c coefficients from the Kp, Ki & Kd */
	    Diag_Return_PRC_S31();
	}
	else
	{
		Diag_Return_NRC_IMLOIF();
	}
}

static void Diag_RC_S31_0004 (void)
{
	if (App_DiagComDrv_UART1RXBuf[0] == 5)
	{
        PIDInit(&App_PumpControl_PIDStc); //This clears the controller history
        App_PumpControl_kCoeffs[0] = (App_DiagComDrv_UART1RXBuf[4] | (App_DiagComDrv_UART1RXBuf[5] << 8)); //Load the P - coefficient
        PIDCoeffCalc(&App_PumpControl_kCoeffs[0], &App_PumpControl_PIDStc); /*Derive the a,b, & c coefficients from the Kp, Ki & Kd */
	    Diag_Return_PRC_S31();
	}
	else
	{
		Diag_Return_NRC_IMLOIF();
	}
}

static void Diag_RC_S31_0005 (void)
{
	if (App_DiagComDrv_UART1RXBuf[0] == 5)
	{
        PIDInit(&App_PumpControl_PIDStc); //This clears the controller history
        App_PumpControl_kCoeffs[1] = (App_DiagComDrv_UART1RXBuf[4] | (App_DiagComDrv_UART1RXBuf[5] << 8)); //Load the I - coefficient
        PIDCoeffCalc(&App_PumpControl_kCoeffs[0], &App_PumpControl_PIDStc); /*Derive the a,b, & c coefficients from the Kp, Ki & Kd */
	    Diag_Return_PRC_S31();
	}
	else
	{
		Diag_Return_NRC_IMLOIF();
	}
}

static void Diag_RC_S31_0006 (void)
{
	if (App_DiagComDrv_UART1RXBuf[0] == 5)
	{
        PIDInit(&App_PumpControl_PIDStc); //This clears the controller history
        App_PumpControl_kCoeffs[2] = (App_DiagComDrv_UART1RXBuf[4] | (App_DiagComDrv_UART1RXBuf[5] << 8)); //Load the D - coefficient
        PIDCoeffCalc(&App_PumpControl_kCoeffs[0], &App_PumpControl_PIDStc); /*Derive the a,b, & c coefficients from the Kp, Ki & Kd */
	    Diag_Return_PRC_S31();
	}
	else
	{
		Diag_Return_NRC_IMLOIF();
	}
}

static void Diag_RC_S31_000A (void)
{
	if (App_DiagComDrv_UART1RXBuf[0] == 5)
	{
        //Compute the new calibration factor
        CAL_CleanAirPumpRatio = __builtin_divsd((long)25400000, App_Main_I_pump);
	    Diag_Return_PRC_S31();
	}
	else
	{
		Diag_Return_NRC_IMLOIF();
	}
}

static void Diag_Return_PRC_S22 (unsigned int DataH, unsigned int DataL)
{
	App_DiagComDrv_UART1TXBuf[0] = 0x7;
	App_DiagComDrv_UART1TXBuf[1] = 0x62;
	App_DiagComDrv_UART1TXBuf[2] = App_DiagComDrv_UART1RXBuf[2];
	App_DiagComDrv_UART1TXBuf[3] = App_DiagComDrv_UART1RXBuf[3];
	App_DiagComDrv_UART1TXBuf[4] = (unsigned char)(DataH);
	App_DiagComDrv_UART1TXBuf[5] = (unsigned char)(DataH >> 8);
	App_DiagComDrv_UART1TXBuf[6] = (unsigned char)(DataL);
	App_DiagComDrv_UART1TXBuf[7] = (unsigned char)(DataL >> 8);
}

static void Diag_Return_PRC_S2F (void)
{
	App_DiagComDrv_UART1TXBuf[0] = 0x3;
	App_DiagComDrv_UART1TXBuf[1] = 0x6F;
	App_DiagComDrv_UART1TXBuf[2] = App_DiagComDrv_UART1RXBuf[2];
	App_DiagComDrv_UART1TXBuf[3] = App_DiagComDrv_UART1RXBuf[3];
	App_DiagComDrv_UART1TXBuf[4] = 0xAA;
	App_DiagComDrv_UART1TXBuf[5] = 0xAA;
	App_DiagComDrv_UART1TXBuf[6] = 0xAA;
	App_DiagComDrv_UART1TXBuf[7] = 0xAA;
}

static void Diag_Return_PRC_S31 (void)
{
	App_DiagComDrv_UART1TXBuf[0] = 0x4;
	App_DiagComDrv_UART1TXBuf[1] = 0x71;
	App_DiagComDrv_UART1TXBuf[2] = App_DiagComDrv_UART1RXBuf[2];
	App_DiagComDrv_UART1TXBuf[3] = App_DiagComDrv_UART1RXBuf[3];
	App_DiagComDrv_UART1TXBuf[4] = 0xAA;
	App_DiagComDrv_UART1TXBuf[5] = 0xAA;
	App_DiagComDrv_UART1TXBuf[6] = 0xAA;
	App_DiagComDrv_UART1TXBuf[7] = 0xAA;
}

static void Diag_Return_NRC_SNS (void)
{
	App_DiagComDrv_UART1TXBuf[0] = 0x3;
	App_DiagComDrv_UART1TXBuf[1] = 0x7F;
	App_DiagComDrv_UART1TXBuf[2] = App_DiagComDrv_UART1RXBuf[1];
	App_DiagComDrv_UART1TXBuf[3] = 0x11;
	App_DiagComDrv_UART1TXBuf[4] = 0xAA;
	App_DiagComDrv_UART1TXBuf[5] = 0xAA;
	App_DiagComDrv_UART1TXBuf[6] = 0xAA;
	App_DiagComDrv_UART1TXBuf[7] = 0xAA;
}

static void Diag_Return_NRC_IMLOIF (void)
{
	App_DiagComDrv_UART1TXBuf[0] = 0x3;
	App_DiagComDrv_UART1TXBuf[1] = 0x7F;
	App_DiagComDrv_UART1TXBuf[2] = App_DiagComDrv_UART1RXBuf[1];
	App_DiagComDrv_UART1TXBuf[3] = 0x13;
	App_DiagComDrv_UART1TXBuf[4] = 0xAA;
	App_DiagComDrv_UART1TXBuf[5] = 0xAA;
	App_DiagComDrv_UART1TXBuf[6] = 0xAA;
	App_DiagComDrv_UART1TXBuf[7] = 0xAA;
}

static void Diag_Return_NRC_ROOR (void)
{
	App_DiagComDrv_UART1TXBuf[0] = 0x3;
	App_DiagComDrv_UART1TXBuf[1] = 0x7F;
	App_DiagComDrv_UART1TXBuf[2] = App_DiagComDrv_UART1RXBuf[1];
	App_DiagComDrv_UART1TXBuf[3] = 0x31;
	App_DiagComDrv_UART1TXBuf[4] = 0xAA;
	App_DiagComDrv_UART1TXBuf[5] = 0xAA;
	App_DiagComDrv_UART1TXBuf[6] = 0xAA;
	App_DiagComDrv_UART1TXBuf[7] = 0xAA;
}

