//****************************************************************************//
//              Define Applicaton SW global variables:                        //
//****************************************************************************//

//Header for DSP functions:
#include <dsp.h>


//------------------
//Module App_Main:
//------------------
    //Nernst cell voltage [0,1mV/bit]
    unsigned int App_Main_V_Nernst;
    //Nernst cell voltage at Ri measurement high pulse [0,1mV/bit]
    unsigned int App_Main_V_RiH;
    //Nernst cell voltage at Ri measurement low pulse [0,1mV/bit]
    unsigned int App_Main_V_RiL;
    //Ipump measured (feedback) [1uA/bit]
    int App_Main_I_pump;
    //Supply voltage [1 mV/bit]
    unsigned int App_Main_V_Supply;


//------------------
//Module App_ISR_T1:
//------------------
    //Actual stage of OS
    unsigned char App_ISR_T1_OSStage;


//------------------
//Module App_OS:
//------------------
    //OS Task active flags
    unsigned char App_OS_TaskActive_1s;
    unsigned char App_OS_TaskActive_4ms;
    unsigned char App_OS_TaskActive_100ms;
    //System time [1s/bit]
    unsigned long App_OS_SystemTime_Sec;


//-------------------------
//Module App_HeaterControl:
//-------------------------
    //Actual state of heater - 0 ... Undefined
    //                         1 ... Cold_Condensation
    //                         2 ... Cold
    //                         3 ... Quite_cold
    //                         4 ... Quite_warm
    //                         5 ... PID_Closed
    //                         6 ... Overheat
    unsigned char App_HeaterControl_HeaterState;
    //Flag identifies whether the operating point of heater is reached
    unsigned char App_HeaterControl_OpPointReached;
    //Target heater voltage computed by heater controller
    unsigned int  App_HeaterControl_DesiredVoltage;
    //Target heater duty cycle computed by heater controller
    unsigned int  App_HeaterControl_DesiredPWMDutyCycle;
    //Data used by heater PID controller
    tPID App_HeaterControl_PIDStc;
    fractional App_HeaterControl_abcCoefficient[3] __attribute__ ((space(xmemory),far));
    fractional App_HeaterControl_controlHistory[3] __attribute__ ((space(ymemory),far));
    fractional App_HeaterControl_kCoeffs[3];

//--------------------
//Module App_DiagComDrv:
//--------------------
    unsigned int App_DiagComDrv_UART1TXBuf[8];
    unsigned int App_DiagComDrv_UART1RXBuf[8];
    //IOCBI mode enable for heater PWM duty cycle: [0x55AA - enable; other - disable]
    unsigned int App_DiagComDrv_IOCBI_Enable_HeaterPWMDutyCycle;
    //Desired heater PWM duty cycle in IOCBI mode [0.01%/bit]
    unsigned int App_DiagComDrv_IOCBI_Data_HeaterPWMDutyCycle;
    //IOCBI mode enable for heater PID target temperature: [0x55AA - enable; other - disable]
    unsigned int App_DiagComDrv_IOCBI_Enable_HeaterPID_TargetTemp;
    //Desired heater PID target temperature in IOCBI mode [0.1C/bit]
    unsigned int App_DiagComDrv_IOCBI_Data_HeaterPID_TargetTemp;
    //IOCBI mode enable for target pump current: [0x55AA - enable; other - disable]
    unsigned int App_DiagComDrv_IOCBI_Enable_U_pump_Target;
    //Desired target pump current in IOCBI mode
    unsigned int App_DiagComDrv_IOCBI_Data_U_pump_Target;
    //IOCBI mode enable for output DAC: [0x55AA - enable; other - disable]
    unsigned int App_DiagComDrv_IOCBI_Enable_U_OutDAC;
    //Desired output DAC voltage in IOCBI mode
    unsigned int App_DiagComDrv_IOCBI_Data_U_OutDAC;



//------------------
//Module App_RiCalc:
//------------------
    //Actual Nernst cell internal resistance [0.1 Ohm/bit]
    unsigned int App_RiCalc_RiFilt;

//---------------------
//Module App_TempCalc:
//---------------------
    //Actual sensor cell temperature [0.1C/bit]
    int App_TempCalc_CellTemp;


//----------------------
//Module App_PumpControl:
//----------------------
    //Ipump computed (target) [1uA/bit]
    unsigned int App_PumpControl_U_pump_Target;
    unsigned char App_PumpControl_OpPointReached;
    //Data used by pump PID controller
    tPID App_PumpControl_PIDStc;
    fractional App_PumpControl_abcCoefficient[3] __attribute__ ((space(xmemory),far));
    fractional App_PumpControl_controlHistory[3] __attribute__ ((space(ymemory),far));
    fractional App_PumpControl_kCoeffs[3];


//----------------------
//Module App_AFRLambdaCalc:
//----------------------
    //Detected lamdba [0.0001/bit]
    int App_AFRLambdaCalc_LambdaDetected;
    int App_AFRLambdaCalc_LambdaDetectedFilt;
    int App_AFRLambdaCalc_AFRDetected;
    int App_AFRLambdaCalc_I_pump_corr;