//Clock frequency definition
#define FCY (70000000)

//Device related libraries include
#include <p33EP128GP504.h>
#include <libpic30.h>
#include <dsp.h>


//Used global variables for write:
extern tPID App_HeaterControl_PIDStc;
extern fractional App_HeaterControl_abcCoefficient[3];
extern fractional App_HeaterControl_controlHistory[3];
extern fractional App_HeaterControl_kCoeffs[3];

extern tPID App_PumpControl_PIDStc;
extern fractional App_PumpControl_abcCoefficient[3];
extern fractional App_PumpControl_controlHistory[3];
extern fractional App_PumpControl_kCoeffs[3];

extern unsigned int App_Main_V_Supply;

//External calibrations:
extern const fractional CAL_HeaterPID_Pcoeff;
extern const fractional CAL_HeaterPID_Icoeff;
extern const fractional CAL_HeaterPID_Dcoeff;

extern const fractional CAL_PumpPID_Pcoeff;
extern const fractional CAL_PumpPID_Icoeff;
extern const fractional CAL_PumpPID_Dcoeff;


void App_Init (void)
{
    //Initialize a PID controller for temp control
    App_HeaterControl_PIDStc.abcCoefficients = &App_HeaterControl_abcCoefficient[0]; /*Set up pointer to derived coefficients */
    App_HeaterControl_PIDStc.controlHistory = &App_HeaterControl_controlHistory[0];  /*Set up pointer to controller history samples */
    PIDInit(&App_HeaterControl_PIDStc);  /*Clear the controler history and the controller output */
    App_HeaterControl_kCoeffs[0] = CAL_HeaterPID_Pcoeff; //P - coefficient
    App_HeaterControl_kCoeffs[1] = CAL_HeaterPID_Icoeff; //I - coefficient
    App_HeaterControl_kCoeffs[2] = CAL_HeaterPID_Dcoeff; //D - coefficient
    PIDCoeffCalc(&App_HeaterControl_kCoeffs[0], &App_HeaterControl_PIDStc); /*Derive the a,b, & c coefficients from the Kp, Ki & Kd */

    //Initialize a PID controller for pump control
    App_PumpControl_PIDStc.abcCoefficients = &App_PumpControl_abcCoefficient[0]; /*Set up pointer to derived coefficients */
    App_PumpControl_PIDStc.controlHistory = &App_PumpControl_controlHistory[0];  /*Set up pointer to controller history samples */
    PIDInit(&App_PumpControl_PIDStc);  /*Clear the controler history and the controller output */
    App_PumpControl_kCoeffs[0] = CAL_PumpPID_Pcoeff; //P - coefficient
    App_PumpControl_kCoeffs[1] = CAL_PumpPID_Icoeff; //I - coefficient
    App_PumpControl_kCoeffs[2] = CAL_PumpPID_Dcoeff; //D - coefficient
    PIDCoeffCalc(&App_PumpControl_kCoeffs[0], &App_PumpControl_PIDStc); /*Derive the a,b, & c coefficients from the Kp, Ki & Kd */

    //Initialize a power supply voltage with a temporary value of 12V, until a valid A/D conversion performs
    App_Main_V_Supply = 12000;
}
