//System clock frequency definition
#define FCY (70000000)

//Device related headers
#include <p33EP128GP504.h>
#include <libpic30.h>

//Global variables definition
#include "../App/App_GlobalSignals.h"
//Hardware driver header
#include "../HWD/HWD.h"
//Calibration header
#include "../CAL/App_Calibrations.h"

//External functions prototypes
void App_Init (void);
void App_OS (void);
void App_PumpControl (void);
void App_AFRLambdaCalc (void);
void App_SetDACOut (void);
void App_DiagComDrv (void);
void App_RiCalc (void);
void App_TempCalc (void);
void App_HeaterControl (void);
void App_RoutineControl (void);

int main (void)
{
    //Initialize the MCU HW
    HWD_Init();
    //Initialize Application
    App_Init();

    while(1)
    {
        //Wait for OS stage 1 start (0 us)
        while(App_ISR_T1_OSStage != 1);
        TESTOUT_PIN11 = 1;
        //Convert Nernst voltage
        App_Main_V_Nernst = HWD_ADC1_convert_Nernst();
        //Convert Pump current
        App_Main_I_pump = HWD_ADC1_convert_IPump();
        TESTOUT_PIN11 = 0;

        //Wait for OS stage 2 start (30 us)
        while(App_ISR_T1_OSStage != 2);
        TESTOUT_PIN12 = 1;
        TESTOUT_PIN13 = 1;
        //HW: Start Ri measurement high pulse
        HWD_START_RI_HIGH_PULSE;
        TESTOUT_PIN12 = 0;
        //Run the OS task control needed by following functions
        App_OS();        
        //Compute and apply the target pump current
        App_PumpControl();
        //Compute the actual detected Lambda
        App_AFRLambdaCalc();
        //Set the detected Lambda on DAC output
        App_SetDACOut();
        TESTOUT_PIN13 = 0;

        //Wait for OS stage 3 start (155 us)
        while(App_ISR_T1_OSStage != 3);
        TESTOUT_PIN14 = 1;
        TESTOUT_PIN33 = 1;
        //Measure Nernst voltage while Ri measurement high pulse is applied
        App_Main_V_RiH = HWD_ADC1_convert_Nernst();
        TESTOUT_PIN33 = 0;
        //HW: Stop Ri measurement high pulse
        HWD_STOP_RI_HIGH_PULSE;
        //HW: Start Ri measurement low pulse
        HWD_START_RI_LOW_PULSE;
        //Run the diagnostic communication protocol driver 
        App_DiagComDrv();
        TESTOUT_PIN14 = 0;

        //Wait for OS stage 4 start (280 us)
        while(App_ISR_T1_OSStage != 4);
        TESTOUT_PIN15 = 1;
        //Measure Nernst voltage while Ri measurement low pulse is applied
        App_Main_V_RiL = HWD_ADC1_convert_Nernst();
        //HW: Stop Ri measurement low pulse
        HWD_STOP_RI_LOW_PULSE;
        //Calculate Ri
        App_RiCalc();
        //Compute the sensor cell temperature
        App_TempCalc();
        //Compute the target heater voltage and PWM duty cycle
        App_HeaterControl();
        //Routine control
        App_RoutineControl();
        TESTOUT_PIN15 = 0;
        
        //Wait for OS stage 5 start (340 us)
        while(App_ISR_T1_OSStage != 5);
        TESTOUT_PIN10 = 1;
        //HW: Start the heater PWM pulse
        HWD_HeaterPWM_SetPulse(App_HeaterControl_DesiredPWMDutyCycle);
        //Convert supply voltage
        App_Main_V_Supply = HWD_ADC1_convert_Supply();
        TESTOUT_PIN10 = 0;
    }
}