//System clock frequency definition
#define FCY (70000000)

//Device related headers
#include <p33EP128GP504.h>
#include <libpic30.h>
#include <dsp.h>

//Hardware driver header
#include "../HWD/HWD.h"
//Hardware calibrations
#include "../CAL/HW_Calibrations.h"

//External calibrations definition
extern const unsigned int CAL_PumpPID_TargetVoltage;

//Global variables used for read:
extern unsigned int App_Main_V_Nernst;
extern unsigned char App_HeaterControl_OpPointReached;
extern unsigned int App_DiagComDrv_IOCBI_Enable_U_pump_Target;
extern unsigned int App_DiagComDrv_IOCBI_Data_U_pump_Target;

//Global variables used for write:
extern unsigned int App_PumpControl_U_pump_Target;
extern unsigned char App_PumpControl_OpPointReached;

//Prototypes of local functions used in this module:

//Data used by pump PID controller
extern tPID App_PumpControl_PIDStc;
extern fractional App_PumpControl_abcCoefficient[3];
extern fractional App_PumpControl_controlHistory[3];
extern fractional App_PumpControl_kCoeffs[3];


//This function calculates the target pump current using a PID controller
void App_PumpControl (void)
{
    //IOCBI control for pump current active?
    if (App_DiagComDrv_IOCBI_Enable_U_pump_Target == 0x55AA)
    {
        //Yes, use the IOCBI value
        App_PumpControl_U_pump_Target = App_DiagComDrv_IOCBI_Data_U_pump_Target;
        //HW: Apply required pumping current
        HWD_PumpDAC_SetUPump(App_PumpControl_U_pump_Target);
    }
    else
    {
        if (App_HeaterControl_OpPointReached)
        {
            /*Load the target Nernst voltage for PID controller*/
            App_PumpControl_PIDStc.controlReference = CAL_PumpPID_TargetVoltage;
            /*Load the actual measured Nernst voltage*/
            App_PumpControl_PIDStc.measuredOutput = App_Main_V_Nernst;
            /*Run the PID control function loaded from library libdsp-elf.a*/
            PID(&App_PumpControl_PIDStc);
            //Limit the PID output range to match the pump DAC resolution
            if (App_PumpControl_PIDStc.controlOutput < (int)(-2047))
            {
                App_PumpControl_PIDStc.controlOutput = (int)(-2047);
            }
            if (App_PumpControl_PIDStc.controlOutput > (int)(2048))
            {
                App_PumpControl_PIDStc.controlOutput = (int)(2048);
            }
            //Add the offset because of unipolar DAC operation
            App_PumpControl_U_pump_Target = (App_PumpControl_PIDStc.controlOutput + 0x7FF);
            //HW: Apply required pumping current
            HWD_PumpDAC_SetUPump(App_PumpControl_U_pump_Target);
        }
        else
        {
            //HW: Disconnect the pumping current
            HWD_PumpDAC_ResetUPump();
        }       
    }
    //Identify whether the Nernst cell operating point is reached
    if ((App_Main_V_Nernst < 4800) && (App_Main_V_Nernst > 4300))
    {
        App_PumpControl_OpPointReached = 1;
    }
    else
    {
        App_PumpControl_OpPointReached = 0;
    }
}