//System clock frequency definition
#define FCY (70000000)

//Device related headers
#include <p33EP128GP504.h>
#include <libpic30.h>

//Hardware calibration constants
#include "../CAL/HW_Calibrations.h"

//Global variables used by module App_RiCalc for read:
extern unsigned int App_Main_V_Nernst;
extern unsigned int App_Main_V_RiH;
extern unsigned int App_Main_V_RiL;

//Global variables used by module App_RiCalc for write:
extern unsigned int App_RiCalc_RiFilt;


void App_RiCalc (void)
{
    unsigned long numerator;
    unsigned int  denominator1;
    unsigned int  denominator2;
    unsigned int  denominator3;
    unsigned int  loc_Ri;

    if (App_Main_V_RiH < 14500) //Is the Ri-measurement positive pulse smaller than 1450 mV (no ADC overrflow)?
    {
      if (App_Main_V_RiL > 500) //Is the Ri-measurement negative pulse greater than 50 mV (no ADC underflow)?
      {
          //Calculate the Ri using both positive and negative Ri-mesurement pulses
          numerator = ((App_Main_V_RiH - App_Main_V_RiL)*100000);
          denominator1 = __builtin_divud((__builtin_muluu((C_V_Vcc8 - C_V_Virt_GND - App_Main_V_RiH),10000)), C_R_rpH);
          denominator2 = __builtin_divud((__builtin_muluu((C_V_Virt_GND + App_Main_V_RiL),10000)),C_R_L);
          denominator3 = __builtin_divud((__builtin_muluu((C_V_Vcc8 - C_V_Virt_GND - App_Main_V_RiL),1000)),
                        (__builtin_divud(C_R_rp,10)));

          loc_Ri = __builtin_divud(numerator,(denominator1 + denominator2 - denominator3));
      }
      else
      {
          //Calculate the Ri using only positive Ri-mesurement pulse.
          numerator = ((App_Main_V_RiH - App_Main_V_Nernst)*100000);
          denominator1 = __builtin_divud((__builtin_muluu((C_V_Vcc8 - C_V_Virt_GND - App_Main_V_RiH),10000)), C_R_rpH);
          denominator2 = __builtin_divud((__builtin_muluu((C_V_Vcc8 - C_V_Virt_GND - App_Main_V_Nernst),1000)),
                        (__builtin_divud(C_R_rp,10)));

          loc_Ri = __builtin_divud(numerator,(denominator1 - denominator2));
      }
    }
    else
    {
        //Ri is too high and can not be computed due to ADC overflow.
        //Write an error value
        loc_Ri = 65535U;
    }

    //Perform a filter
    static unsigned int loc_RiHistory[10];
    long sum = 0;
    int i;

    for (i = 9; i >= 1; i--)
    {
        loc_RiHistory[i] = loc_RiHistory[i-1];
    }

    loc_RiHistory[0] = loc_Ri;
    
    for (i = 0; i <= 9; i++)
    {
        sum += loc_RiHistory[i];
    }

    App_RiCalc_RiFilt = (__builtin_divsd(sum, 10));
}