//System clock frequency definition
#define FCY (70000000)

//Device related headers
#include <p33EP128GP504.h>
#include <libpic30.h>
#include <dsp.h>

//Hardware driver header
#include "../HWD/HWD.h"

#define DAC_MODE 1 //0 For narrowband, 1 For wideband


//Global variables used for read:
extern int App_AFRLambdaCalc_LambdaDetected;
extern unsigned char App_HeaterControl_HeaterState;
extern unsigned int App_DiagComDrv_IOCBI_Enable_U_OutDAC;
extern unsigned int App_DiagComDrv_IOCBI_Data_U_OutDAC;

//Global variables used for write:


//This function scales and puts the detected Lambda factor to DAC output
void App_SetDACOut (void)
{
    //Create a temporary signal
    int loc_DACOutVoltage;
    
    #if (DAC_MODE == 1)
    //Voltage output
    //Warming-up phase in progress?
    if (App_AFRLambdaCalc_LambdaDetected == -1)
    {
        //Yes, then result is not available -> output = 100 mV
        loc_DACOutVoltage = 100;
    }
    else
    {
        //Sensor overheat?
        if (App_AFRLambdaCalc_LambdaDetected == -2)
        {
            //Yes, output 200 mV
            loc_DACOutVoltage = 200;
        }
        else
        {
            //Temperature operation point not reached?
            if (App_AFRLambdaCalc_LambdaDetected == -3)
            {
                //Yes, output 300 mV
                loc_DACOutVoltage = 300;
            }
            else
            {
                //Pump operation point not reached?
                if (App_AFRLambdaCalc_LambdaDetected == -4)
                {
                    //Yes, output 400 mV
                    loc_DACOutVoltage = 400;
                }
                else
                {
                    //Not enough power supply voltage?
                    if (App_AFRLambdaCalc_LambdaDetected == -5)
                    {
                        //Yes, output 500 mV
                        loc_DACOutVoltage = 500;
                    }
                    else
                    {
                        //Is the Lambda > 3.5?
                        if (App_AFRLambdaCalc_LambdaDetected > 3500)
                        {
                            //Yes, output 3500 mV to prevent DAC overflow
                            loc_DACOutVoltage = 3500;
                        }
                        else
                        {
                            //Ok, set the detected lambda to DAC
                            loc_DACOutVoltage = App_AFRLambdaCalc_LambdaDetected;
                        }
                    }
                }
            }
        }
    }
    #else
    //Narrow band lambda emulation output
    if (App_AFRLambdaCalc_LambdaDetected < 0xFFFC)
    {
        if (App_AFRLambdaCalc_LambdaDetected < 1000)
        {
            loc_DACOutVoltage = 990;
        }
        else
        {
            loc_DACOutVoltage = 90;
        }
            
    }
    else
    {
        loc_DACOutVoltage = 500;
    }
    #endif
    
    //IOCBI mode active?
    if (App_DiagComDrv_IOCBI_Enable_U_OutDAC == 0x55AA)
    {
        //Yes, then take a data from IOCBI signal
        loc_DACOutVoltage = App_DiagComDrv_IOCBI_Data_U_OutDAC;
    }
    //Add a trim offset and call the HW driver to set the output DAC voltage
    HWD_OUTDAC_SetVoltage(loc_DACOutVoltage);   
}