//System clock frequency definition
#define FCY (70000000)

//Device related headers
#include <p33EP128GP504.h>
#include <libpic30.h>


//Global variables used by module App_TempCalc for read:
extern unsigned int App_RiCalc_RiFilt;

//Global variables used by module App_TempCalc for write:
extern int App_TempCalc_CellTemp;

//External Calibrations definitions:
extern const int CAL_TBL_Ri_Temp[2][12];

//Externally used functions prototypes
extern int App_Interpolation_Linear_si16 (int x, int size, const int array[2][size/2]);

//This function calculates the Heater temperature from Nernst cell internal resistance
void App_TempCalc (void)
{
    //Is the sensor cell hot enough to be the temperature measurable?
    if (App_RiCalc_RiFilt < 10000) //1kOhm
    {
        //Yes, then calculate the temperature using Ri/Temp characteristic table
        App_TempCalc_CellTemp = App_Interpolation_Linear_si16 (App_RiCalc_RiFilt, (sizeof(CAL_TBL_Ri_Temp) / sizeof(CAL_TBL_Ri_Temp[0][0])), CAL_TBL_Ri_Temp);
    }
    else
    {
        //No, then output = 0xFFFF, that means the temperature is not measurable
        App_TempCalc_CellTemp = 0xFFFF;
    }
}