#include <dsp.h> //dsp.h for usage Q15 format


//----------------------------------------------------------------------------//
//                 Application software calibration definitions               //
//----------------------------------------------------------------------------//

//Clean air pump current ratio
    int CAL_CleanAirPumpRatio = 11766;// [0.0001/bit] 
    int CAL_SensorParasiticOffset = 15;// [0.0001/bit] 

//Length of OS stages 1,2,3,4. [T1-ticks / 14.2857 ns]
    const unsigned int CAL_OSStagesLength[5] = {2100, 8750, 8750, 4200, 4200}; //{30us, 125us, 125us, 60us, 60 us}

//Ri vs. Nernst cell temperature chracteristic table
    const int CAL_TBL_Ri_Temp[2][12] = {{  800,   900,  1000,  2000,  3000,  4000,  5000,  6000,  7000,  8000,  9000, 10000}, //RiN [0.1 Ohm/bit]
                                        {10570, 10220,  9900,  8430,  7800,  7380,  7100,  6880,  6700,  6550,  6400,  6300}}; //Cell temperature [C]


//Heater states thresholds
    const unsigned int CAL_Ri_PID_CLOSED_Lth = 3000; //[0.1 Ohm/bit]
    const unsigned int CAL_Ri_PID_CLOSED_Uth = 9500; //[0.1 Ohm/bit]
    const unsigned int CAL_Ri_OVERHEAT_Lth = 2000; //[0.1 Ohm/bit]
    const unsigned int CAL_Ri_OVERHEAT_Uth = 3500; //[0.1 Ohm/bit]

//Condensation phase duration
    const unsigned int CAL_t_Condensation = 5; //[1 sec/bit]

//Condensation phase heater voltage
    const unsigned int CAL_U_Condensation = 2000; //[1mV/bit]

//Initial heater voltage when cold
    const unsigned int CAL_U_Initial_Cold = 4000; //[1mV/bit]

//Heater voltage increase step per 100ms when cold
    const unsigned int CAL_dU_Cold = 20; //[1mV/bit]

//Heater voltage increase step per 100ms when quite cold
    const unsigned int CAL_dU_QuiteCold = 15; //[1mV/bit]

//Heater voltage increase step per 100ms when quite warm
    const unsigned int CAL_dU_QuiteWarm = 9; //[1mV/bit]

//Heater PID controller parameters
    const fractional   CAL_HeaterPID_Pcoeff = 25000; //0.76294
    const fractional   CAL_HeaterPID_Icoeff = 250; //0.00762
    const fractional   CAL_HeaterPID_Dcoeff = 0;

//Heater PID controller setpoint temperature
    const int          CAL_HeaterPID_TargetTemp = 7800; //[0.1C/bit]

//Pump PID controller parameters
    const fractional   CAL_PumpPID_Pcoeff = 30000; //0.91553
    const fractional   CAL_PumpPID_Icoeff = 30000; //0.91553
    const fractional   CAL_PumpPID_Dcoeff = 24500; //0.74769

//Pump PID controller setpoint voltage
    const unsigned int CAL_PumpPID_TargetVoltage = 4500; //[0.1mV/bit]

//Pump current vs. Lambda look-up table
    const int CAL_TBL_Ip_Lambda[2][24] =
   {{ -2000, -1602, -1243,  -927,  -800,  -652,  -405,  -183,  -106,   -40,     0,    15,    97,   193,   250,   329,   671,   938,  1150,  1385,  1700,  2000,  2150,  2250},  //Ip [uA]
    {   650,   700,   750,   800,   822,   850,   900,   950,   970,   990,  1003,  1010,  1050,  1100,  1132,  1179,  1429,  1701,  1990,  2434,  3413,  5391,  7506, 10119}};  //Lambda [-/1000]

//Pump current vs. AFR for gasoline look-up table
    const int CAL_TBL_Ip_AFR[2][24] =
   {{ -2000, -1602, -1243,  -927,  -800,  -652,  -405,  -183,  -106,   -40,     0,    15,    97,   193,   250,   329,   671,   938,  1150,  1385,  1700,  2000,  2150,  2250},  //Ip [uA]
    {  9542, 10276, 11010, 11744, 12067, 12478, 13212, 13946, 14240, 14533, 14724, 14826, 15414, 16148, 16618, 17307, 20997, 24971, 29213, 32767, 32767, 32767, 32767, 32767}}; //AFR [-/1000]