/*****************************************************************************/
// Hardware driver for:
// Output pin control for generation Ri measurement pulses
// ADC sampling and conversion of Nernst voltage and Pump current
// DAC conversions for generation output AFR signal
// DAC conversions for generation of Ipump
// Output compare for Heater PWM generation
/*****************************************************************************/

//Device related headers
#include <p33EP128GP504.h>


/***************Used Global Variables definition***********/

/***************Global HW driver definitions***************/
//Chip select signals for SPI communication
#define CS_ACTIVE 0
#define CS_INACTIVE 1
#define U1RX_INTERRUPT_ENABLE (IEC0bits.U1RXIE = 1)
#define U1RX_INTERRUPT_DISABLE (IEC0bits.U1RXIE = 0)

/***************I/O pins definitions***********************/
#define SPI2_OUTDAC_CS LATBbits.LATB7
#define SPI2_IPumpDAC_CS LATBbits.LATB8

#define SPARE_OUT LATCbits.LATC6

#define TESTOUT_PIN10 LATBbits.LATB12
#define TESTOUT_PIN11 LATBbits.LATB13
#define TESTOUT_PIN12 LATAbits.LATA10
#define TESTOUT_PIN13 LATAbits.LATA7
#define TESTOUT_PIN14 LATBbits.LATB14
#define TESTOUT_PIN15 LATBbits.LATB15
#define TESTOUT_PIN33 LATBbits.LATB4

/*******Macros for output pins control - Ri measurement pulses generation******/
#define HWD_START_RI_HIGH_PULSE LATCbits.LATC7 = 1
#define HWD_STOP_RI_HIGH_PULSE LATCbits.LATC7 = 0
#define HWD_START_RI_LOW_PULSE LATCbits.LATC8 = 1
#define HWD_STOP_RI_LOW_PULSE LATCbits.LATC8 = 0

/***************HWD function prototypes***********/
void HWD_Init (void);
int HWD_ADC1_convert_Nernst (void);
int HWD_ADC1_convert_IPump (void);
int HWD_ADC1_convert_Supply (void);
void HWD_OUTDAC_SetVoltage (unsigned int loc_Voltage);
void HWD_OUTDAC_ResetVoltage (void);
void HWD_PumpDAC_SetUPump (unsigned int x);
void HWD_PumpDAC_ResetUPump (void);
void HWD_HeaterPWM_SetPulse (unsigned int loc_DutyCycle);
unsigned char HWD_CheckUARTRXBuffer (void);

