// MCU configuration bits settings:

// FICD
#pragma config ICS = PGD1               // ICD Communication Channel Select bits (Communicate on PGEC1 and PGED1)
#pragma config JTAGEN = OFF             // JTAG Enable bit (JTAG is disabled)

// FPOR
#pragma config ALTI2C1 = OFF            // Alternate I2C1 pins (I2C1 mapped to SDA1/SCL1 pins)
#pragma config ALTI2C2 = OFF            // Alternate I2C2 pins (I2C2 mapped to SDA2/SCL2 pins)
#pragma config WDTWIN = WIN25           // Watchdog Window Select bits (WDT Window is 25% of WDT period)

// FWDT
#pragma config WDTPOST = PS32768        // Watchdog Timer Postscaler bits (1:32,768)
#pragma config WDTPRE = PR128           // Watchdog Timer Prescaler bit (1:128)
#pragma config PLLKEN = ON              // PLL Lock Enable bit (Clock switch to PLL source will wait until the PLL lock signal is valid.)
#pragma config WINDIS = OFF             // Watchdog Timer Window Enable bit (Watchdog Timer in Non-Window mode)
#pragma config FWDTEN = OFF             // Watchdog Timer Enable bit (Watchdog timer enabled/disabled by user software)

// FOSC
#pragma config POSCMD = NONE            // Primary Oscillator Mode Select bits (Primary Oscillator disabled)
#pragma config OSCIOFNC = ON            // OSC2 Pin Function bit (OSC2 is general purpose digital I/O pin)
#pragma config IOL1WAY = ON             // Peripheral pin select configuration (Allow only one configuration)
#pragma config FCKSM = CSECMD           // Clock Switching Mode bits (Clock switching is enabled,Fail-safe Clock Monitor is disabled)

// FOSCSEL
#pragma config FNOSC = FRC              // Oscillator Source Selection (Internal Fast RC (FRC))
#pragma config IESO = OFF               // Two-speed Oscillator Start-up Enable bit (Start up with user-selected oscillator source)

// FGS
#pragma config GWRP = OFF               // General Segment Write-Protect bit (General Segment may be written)
#pragma config GCP = OFF                // General Segment Code-Protect bit (General Segment Code protect is Disabled)

//Clock frequency definition
#define FCY (70000000)

//Device related headers
#include <p33EP128GP504.h>
#include <libpic30.h>

extern unsigned int App_DiagComDrv_UART1TXBuf[8];
extern unsigned int App_DiagComDrv_UART1RXBuf[8];


void HWD_Init (void)
{
    // Oscillator config, switch to FRC + PLL 140 MHz / 70 MIPS
    PLLFBDbits.PLLDIV    = 74; //74 for 70.015 MIPS , 63 for 59.881250 MIPS
    CLKDIVbits.PLLPOST   =  0;
    CLKDIVbits.PLLPRE    =  0;
    __builtin_write_OSCCONH (0x01);
    __builtin_write_OSCCONL (OSCCON | 0x01);
    while (OSCCONbits.COSC != 0b001);
    while (OSCCONbits.LOCK != 1);

    //PORTA config
    // RA0/PIN19/AN0 - Analog IN, AN channel 0 -> Nernst voltage;
    // RA1/PIN20/AN1 - Analog IN; AN channel 1 -> Pump current;
    // RA7/PIN13 - digital out - TESTOUT_PIN13;
    // RA10/PIN12 - digital out - TESTOUT_PIN12;
    // *Unused GPIO pins configured as outputs
    ANSELA = 0b0000000000000011;
    TRISA = 0b0000000000000011;

    //PORTB config
    // RB0/PIN21/AN2 - Analog IN; AN channel 2 -> Supply voltage;
    // RB1/PIN22/AN3 - Not used
    // RB2/PIN23 - Debug port PGEC1
    // RB3/PIN24 - Debug port PGED1
    // RB4/PIN33/RP36 - digital out - TESTOUT_PIN33
    // RB5/PIN41/RP37 - alternate function - SPI2 SDO2 OUT - AFR output DAC and Ipump DAC
    // RB6/PIN42/RP38 - alternate function - SPI2 SCK2 OUT - AFR output DAC and Ipump DAC
    // RB7/PIN43/RP39 - digital OUT -> CS for AFR output DAC;
    // RB8/PIN44 - digital OUT -> CS for Ipump DAC;
    // RB9/PIN1/RP41 - alternate function OC1 OUT -> Heater PWM
    // RB10/PIN8/RP42 - alternate function UART1RX IN
    // RB11/PIN9/RP43 - alternate function UART1TX OUT

    ANSELB = 0b0000000000000001;
    TRISB = 0b0000010000000001;
    RPINR18bits.U1RXR = 42;  //UART1RX IN
    RPOR4bits.RP43R   = 1;   //UART1TX OUT
    RPOR3bits.RP41R   = 16;  //OC1 OUT
    RPOR1bits.RP37R   = 8;   //SPI2 SDO2 OUT
    RPOR2bits.RP38R   = 9;   //SPI2 SCK2 OUT

    //PORTC config
    // RC6/PIN2/RP54 - Digital OUT -> Spare OUT
    // RC7/PIN3/RP55 - Digital OUT -> Ri measurement positive pulse
    // RC8/PIN4/RP56 - Digital OUT -> Ri measurement negative pulse
    ANSELC = 0;
    TRISC = 0b0000000000000000;

    // Lock the peripheral pin select
    __builtin_write_OSCCONL (OSCCON | 0b0000000001000000);


    // ADC1 module config
    AD1CON3bits.ADCS   = 8;       // ADC clock prescaler: Tad = Tcy*(ADCS+1)
    AD1CON1bits.AD12B  = 1;       // 12-bit mode
    AD1CHS0bits.CH0SA  = 1;       // CH0 Convert AN1 input - Nerst voltage
    AD1CON1bits.ASAM   = 0;       // No auto-sampling
    AD1CON1bits.SSRC   = 0b111;   // Sampling duration controlled by internal counter
    AD1CON3bits.SAMC   = 10;       // Sampling duration 10 TAD
    AD1CON1bits.ADON   = 1;       // Turn on the ADC module
    

    //DMA0 config - UART1 TX
    DMA0CONbits.DIR = 1; // RAM-to-Peripheral
    DMA0CONbits.SIZE = 0; // Word transfer
    DMA0CONbits.MODE = 1; // One shot mode
    DMA0CNT = 7; // Eight DMA requests
    DMA0REQ = 0x000C; // Select UART1 transmitter
    DMA0PAD = (volatile unsigned int) &U1TXREG;
    DMA0STAL = __builtin_dmaoffset(App_DiagComDrv_UART1TXBuf);
    DMA0STAH = 0x0000;
    IFS0bits.DMA0IF = 0; // Clear DMA Interrupt Flag
    IEC0bits.DMA0IE = 0; // DMA interrupt
    IPC1bits.DMA0IP = 1;
    
    

    //DMA1 config - UART1 RX:
    DMA1CONbits.DIR = 0; // Peripheral-to-RAM
    DMA1CONbits.SIZE = 0; // Word transfer
    DMA1CONbits.MODE = 0; // Continuous mode
    DMA1CNT = 7; // Eight DMA requests
    DMA1REQ = 0x000B; // Select UART1 receiver
    DMA1PAD = (volatile unsigned int) &U1RXREG;
    DMA1STAL = __builtin_dmaoffset(App_DiagComDrv_UART1RXBuf);
    DMA1STAH = 0x0000;
    IFS0bits.DMA1IF = 0; 
    IEC0bits.DMA1IE = 0;
    IPC3bits.DMA1IP = 1;
    DMA1CONbits.CHEN = 1;


    // UART1 config - Used for main communication interface
    //U1BRG = ((FCY/BAUDRATE)/4) - 1
    U1BRG             = 1822;
    U1MODEbits.BRGH   = 1;
    U1STAbits.URXISEL = 0;
    U1MODEbits.UARTEN = 1;
    U1STAbits.UTXEN   = 1;
    IPC2bits.U1RXIP    = 1; // Priority lowest
    IFS0bits.U1RXIF    = 0; // Clear interrupt flag
    IEC0bits.U1RXIE    = 0; // Interrupt disabled


    // SPI2 config -> Analog Output DAC + Ipump control DAC
    SPI2CON1bits.MODE16 = 1;      // 16-bit SPI communication
    SPI2CON1bits.MSTEN  = 1;      // Master mode
    SPI2CON1bits.CKE    = 1;      // Data sampling at rising edge of SCK
    SPI2CON1bits.PPRE   = 3;      // Primary prescaler 1:1
    SPI2CON1bits.SPRE   = 1;      // Secondary prescaler 7:1 -> 10 MHz clock
    SPI2STATbits.SPIEN  = 1;      // Turn on the SPI2 module

    // Output Compare1 config - Used for heater PWM (one-pulse mode output compare)
    OC1CON1bits.OCTSEL  = 0b111;      // 7 = peripheral clock, T = 14.28 ns
    OC1CON2bits.SYNCSEL = 0b00000;    // No sync/trig
    
    __delay_ms(100); //Wait for power supply to stabilize
    
    //Reset the UART RX DMA channel to flush the received data 
    DMA1CONbits.CHEN = 0;
    DMA1CONbits.CHEN = 1;

    // Timer1 config - used for OS timing
    PR1                 = 1400;   // Timer1 period value = 20 us -> Init task time;
    T1CONbits.TCKPS     = 0;      // 0 = prescaler 1:1 = peripheral clock 14.28 ns
    T1CONbits.TON       = 1;      // Timer on - Init task time window starts
    IPC0bits.T1IP       = 2;      // Interrupt priority
    IFS0bits.T1IF       = 0;      // Clear interrupt flag
    IEC0bits.T1IE       = 1;      // Enable the interrupt

    // Global Interrupt priority
    SRbits.IPL         = 0;

    // Global Interrupt enable
    INTCON2bits.GIE    = 1;

}
